//
// Retro system pause with fadein/fadeout effect
//
// Author     : fabian@lucchi.me
// MysticBBS  : v1.12 alpha 31
//
// 2017.01.04 v1.0   First release
// 2017.01.17 v1.0a  Convert all chars to lowercase to prevent OS problems (thx xqtr)
//

Uses
  CFG;

Const
  csiMaxFrames      = 50; // Maximum frames in loaded anim files
  csiDefaultTimeout = 30; // 30 seconds
  cssDefaultPath    = 'custompauses/';

  cssFile_Anim      = 'mypause.txt'; // main pause message animation
  cssFile_FadeIn    = 'fadein.txt';  // animation for FadeIn effect
  cssFile_FadeOut   = 'fadeout.txt'; // animation for FadeOut effect

Var
  iDelay     : Integer;
  asAnim     : Array[1..csiMaxFrames] Of String;
  iLines     : Integer;
  lTimer     : LongInt;
  bTimeout   : Boolean = False;

//
// LoadAnim reads a given file in memory (array of string)
// and sets the number of available lines. It also read the
// default timeout that is the first line of the file
//
Procedure LoadAnim(sFilename : String);
Var
  tfInput : File;
  sLine   : String;
Begin
  fAssign(tfInput, CfgMpePath + cssDefaultPath + sFileName, 2);
  fReset(tfInput);
  
  // First line is the delay between frame (ie. Delay time in ms)
  fReadLn(tfInput, sLine);
  iDelay := Str2Int(sLine);

  // Load anim strings
  iLines := 0;
  Repeat
    iLines := iLines + 1;
    fReadLn(tfInput, asAnim[iLines]);
  Until fEOF(tfInput) Or (iLines>=csiMaxframes);

  fClose(tfInput);
End; // Procedure LoadAnim

//
// CheckTimeout set a global flag bTimeout if a timeout has occured
//
Procedure CheckTimeout;
Var
  lLocal : LongInt;
Begin
  lLocal := Timer - lTimer;
  If lLocal>csiDefaultTimeout Then Begin
    bTimeout := True;
  End;
End;

//
// Standard procedure to play animation files
// bWait if True asks for a Keypress and process a timeout
//
Procedure PlayAnim(bWait : Boolean);
Var
  iX, iY  : Integer;
  iCount  : Integer;
Begin
  iX := 1;
  iY := WhereY;

  Case bWait Of
    True : Begin
             // Display anim and wait for keypress or timeout
             iCount := 1;
             Repeat
               GotoXY(iX, iY);
               Write(asAnim[iCount]);

               iCount := iCount + 1;
               If iCount>iLines Then iCount := 1;

               // Wait for keypress and process any timeout delay
               Delay(iDelay);
               CheckTimeout;
             Until Keypressed Or bTimeout;
           End;
    False : Begin
	      // Display anim and don't wait for any keytroke
              For iCount := 1 To iLines Do Begin
                GotoXY(iX, iY);
                Write(asAnim[iCount]);
                Delay(iDelay);
              End; // For iCount
            End;
  End; // Case bWait
End;

//
// Init, display basic stuff is no ANSI abilities on remote user
// Load fadein, play it, load main anim, play it, load fadeout, playit
//
Begin
  lTimer := Timer;

  If Graphics = 0 Then Begin
    // No ansi on remote end :(
    Write('-=[ Please press any key ]=-');
    Repeat
      CheckTimeout;
      Delay(100); // 100ms
    Until Keypressed Or bTimeout;
    Halt;
  End; // End of no ansi alternative

  // Load Fadein anim
  LoadAnim(cssFile_FadeIn);
  PlayAnim(False);

  // Read anim file
  LoadAnim(cssFile_Anim);
  PlayAnim(True); // Play anim with waiting for keypress & timeout

  LoadAnim(cssFile_FadeOut);
  PlayAnim(False); // Play anim without waiting for anything

  GotoXY(1, WhereY);
  ClrEol;

  StuffKey(Chr(13)); // Still not clear why this step has to be programmed. Once the paus
                     // is done, it should go to next men but during direct play (menu
                     // fonction : GX with FancyPause, it works beautifuly, but when
                     // using system pause (through theme customization, prompts and
                     // editing #022, it asks for a second keypress. Still looking for any reason
End

